import 'package:length_unit/dynamic_offset.dart';
import 'package:length_unit/dynamic_border_radius.dart';
import 'package:length_unit/length_unit.dart';
import 'package:length_unit/dynamic_edge_insets.dart';

Length? parseLength(String? string) {
  return Length.fromJson(string);
}

DynamicOffset? parseDynamicOffset(Map<String, dynamic>? map) {
  if (map == null) return null;
  Length dx = Length.fromJson(map['dx']) ?? Length(0);
  Length dy = Length.fromJson(map['dy']) ?? Length(0);
  return DynamicOffset(dx, dy);
}

DynamicRadius? parseDynamicRadius(Map<String, dynamic>? map) {
  if (map == null) return null;
  Length x = Length.fromJson(map['x']) ?? Length(0);
  Length y = Length.fromJson(map['y']) ?? Length(0);
  return DynamicRadius.elliptical(x, y);
}

DynamicBorderRadius? parseDynamicBorderRadius(Map<String, dynamic>? map) {
  if (map == null) return null;
  DynamicRadius topLeft =
      parseDynamicRadius(map["topLeft"]) ?? DynamicRadius.zero;
  DynamicRadius topRight =
      parseDynamicRadius(map["topRight"]) ?? DynamicRadius.zero;
  DynamicRadius bottomLeft =
      parseDynamicRadius(map["bottomLeft"]) ?? DynamicRadius.zero;
  DynamicRadius bottomRight =
      parseDynamicRadius(map["bottomRight"]) ?? DynamicRadius.zero;
  return DynamicBorderRadius.only(
    topLeft: topLeft,
    topRight: topRight,
    bottomRight: bottomRight,
    bottomLeft: bottomLeft,
  );
}

DynamicEdgeInsets? parseDynamicEdgeInsets(Map<String, dynamic>? map) {
  if (map == null) return null;
  Length? top = parseLength(map['top']);
  Length? bottom = parseLength(map['bottom']);
  Length? left = parseLength(map['left']);
  Length? right = parseLength(map['right']);
  return DynamicEdgeInsets.only(
    top: top,
    bottom: bottom,
    left: left,
    right: right,
  );
}